package org.example.buddy;

import java.util.ArrayList;
import java.util.List;

import org.example.serviciosweb.HiloWeb;
import org.example.serviciosweb.HiloWebMostrarAmigos;
import org.example.serviciosweb.HiloWebMostrarLocalizacion;
import org.example.serviciosweb.ServicioActualizacionListas;
import org.example.serviciosweb.ServicioMensajeria;
import org.example.serviciosweb.TipoOperacion;

import android.app.Activity;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMapClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

public class Home extends FragmentActivity implements OnMapClickListener,
		LocationListener {

	// Usuario que vino del login
	private String usuario;

	// Manejador para la localización
	private LocationManager manejador;

	// Si hay una localización mejor, la guarda
	public static Location mejorLocaliz;

	private GoogleMap mapa;

	// Para actualizar el estado una vez pasa el login
	private String estado = "conectado";

	// Para iniciar las peticiones web
	private HiloWeb hiloWeb;

	// Diálogo que mostraremos mientras el usuario localiza a un amigo
	private ProgressDialog progreso;

	// Variable que guardaremos en el bundle para saber si debemos de seguir
	// mostrando la información cuando gire la pantalla.
	private boolean infVisible;

	// Layout que contiene la ayuda
	private LinearLayout layout_ayuda;

	private NotificationManager nm;

	private static final int ID_NOTIFICACION = 1;

	// Sirve para mostrar los mensajes recibido por los amigos.
	private List<String> mensajes;

	// ListView para mostrar la lista de usuarios conectados.
	private ListView lista_conectados;

	private ListView lista_desconectados;

	private boolean desconecta;

	// Hilos para pedir via web los amigos conectados y desconectados.
	private HiloWebMostrarAmigos hiloWebMostrarAmigosDes;
	private HiloWebMostrarAmigos hiloWebMostrarAmigosCon;

	// BroadCast que recibe las respuestas de los servicios de mensajería y
	// actualización de listas de amigos.
	private BroadcastReceiver broadcastReceiver;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.home_layout);

		// Preparamos el sonido de la notificación
		// player = MediaPlayer.create(this, R.raw.silvido);

		/*
		 * Registramos el receptor de mensajes para mostrar los mensajes en la
		 * barra de notificación(BroadcastReceiver). Puede ser que el receptor
		 * de mensajes ya esté registrado y esto genera excepciones.
		 */
		try {
			IntentFilter filtro = new IntentFilter(ReceptorMensajes.ACTION_RESP);
			filtro.addAction(ReceptorMensajes.ACTION_LISTA);
			filtro.addCategory(Intent.CATEGORY_DEFAULT);
			broadcastReceiver = new ReceptorMensajes();
			registerReceiver(broadcastReceiver, filtro);
		} catch (Exception e) {
			Log.e("Buddy", e.getMessage());
		}

		/*
		 * Recuperamos vistas determinadas para interactuar con ellas.
		 */
		TextView text_nombre_usuario = (TextView) findViewById(R.id.text_nombre_usuario);
		Button bt_add_amigos = (Button) findViewById(R.id.bt_add_amigos);
		Button bt_delete_amigos = (Button) findViewById(R.id.bt_delete_amigos);
		Button bt_invitar_amigos = (Button) findViewById(R.id.bt_invitar_amigos);
		lista_conectados = (ListView) findViewById(R.id.lista_conectados);
		lista_desconectados = (ListView) findViewById(R.id.lista_desconectados);

		List<String> lista = new ArrayList<String>();
		lista.add(Constantes.MSJ_CARGANDO_LISTA);
		ArrayAdapter<String> listaAmigos = new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, lista);

		lista_conectados.setAdapter(listaAmigos);
		lista_desconectados.setAdapter(listaAmigos);

		/*
		 * Recogemos los datos guardados.
		 */
		if (savedInstanceState != null) {
			infVisible = savedInstanceState.getBoolean("infVisible");
			// Carga las listas con usuarios conectados y desconectados.
			cargarListaConectados(savedInstanceState);
			cargarListaDesconectados(savedInstanceState);

		}

		// Sacamos de la actividad ingreso el usuario que hizo login.
		Bundle extras = getIntent().getExtras();
		usuario = extras.getString("usuario");
		text_nombre_usuario.setText(usuario);

		actualizarEstado(mejorLocaliz);

		layout_ayuda = (LinearLayout) findViewById(R.id.layout_ayuda);

		if (infVisible) {
			layout_ayuda.setVisibility(View.VISIBLE);

		} else {
			layout_ayuda.setVisibility(View.GONE);
		}

		lista_conectados.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> l, View v, int position,
					long id) {
				String amigo = l.getAdapter().getItem(position).toString();

				TextView text_accion_amigo = (TextView) findViewById(R.id.text_accion_amigo);

				Button bt_localizar_amigo = (Button) findViewById(R.id.bt_localizar_amigo);

				Button bt_mandar_mensaje = (Button) findViewById(R.id.bt_mandar_mensaje);

				if (!amigo.equals(Constantes.MSJ_CARGANDO_LISTA)
						&& !amigo.equals(Constantes.MSJ_LISTA_NO_USER)
						&& !amigo.equals(Constantes.MSJ_LISTA_FALLO)) {

					// Asigno un tag al botón localizar para luego cuando lo
					// pulse, sepa a qué usuario debe localizar
					bt_localizar_amigo.setTag(amigo);
					bt_mandar_mensaje.setTag(amigo);

					/*
					 * Si pulsamos un elemento de la lista, eliminamos el texto
					 * informativo y hacemos visibles dos botones
					 */
					text_accion_amigo.setVisibility(View.GONE);
					bt_localizar_amigo.setVisibility(View.VISIBLE);
					bt_mandar_mensaje.setVisibility(View.VISIBLE);

				} else {
					text_accion_amigo.setVisibility(View.VISIBLE);
					bt_localizar_amigo.setVisibility(View.GONE);
					bt_mandar_mensaje.setVisibility(View.GONE);
				}

			}

		});

		lista_desconectados.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> l, View v, int position,
					long id) {
				String amigo = l.getAdapter().getItem(position).toString();

				TextView text_accion_amigo = (TextView) findViewById(R.id.text_accion_amigo);

				Button bt_localizar_amigo = (Button) findViewById(R.id.bt_localizar_amigo);

				Button bt_mandar_mensaje = (Button) findViewById(R.id.bt_mandar_mensaje);

				if (!amigo.equals(Constantes.MSJ_CARGANDO_LISTA)
						&& !amigo.equals(Constantes.MSJ_LISTA_NO_USER)
						&& !amigo.equals(Constantes.MSJ_LISTA_FALLO)) {

					// Asigno un tag al botón localizar para luego cuando lo
					// pulse, sepa a qué usuario debe localizar
					bt_localizar_amigo.setTag(amigo);
					bt_mandar_mensaje.setTag(amigo);

					/*
					 * Si pulsamos un elemento de la lista, eliminamos el texto
					 * informativo y hacemos visibles dos botones
					 */
					text_accion_amigo.setVisibility(View.GONE);
					bt_localizar_amigo.setVisibility(View.GONE);
					bt_mandar_mensaje.setVisibility(View.VISIBLE);

				} else {
					text_accion_amigo.setVisibility(View.VISIBLE);
					bt_localizar_amigo.setVisibility(View.GONE);
					bt_mandar_mensaje.setVisibility(View.GONE);
				}

			}

		});

		mostrarAmigos();

		manejador = (LocationManager) getSystemService(LOCATION_SERVICE);

		activarProveedores();

		mapa = ((SupportMapFragment) getSupportFragmentManager()

		.findFragmentById(R.id.map)).getMap();

		mapa.setMapType(GoogleMap.MAP_TYPE_SATELLITE);

		mapa.setMyLocationEnabled(true);

		mapa.getUiSettings().setZoomControlsEnabled(false);

		mapa.getUiSettings().setCompassEnabled(true);

		mapa.setMyLocationEnabled(true);

		mapa.getUiSettings().setZoomControlsEnabled(false);

		mapa.getUiSettings().setCompassEnabled(true);

		mapa.setOnMapClickListener(this);

		manejador = (LocationManager) getSystemService(LOCATION_SERVICE);

		nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

		bt_add_amigos.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				lanzarAddAmigos();
			}
		});

		bt_delete_amigos.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				lanzarDeleteAmigos();
			}
		});

		bt_invitar_amigos.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				lanzarInvitarAmigos();
			}
		});

		/*
		 * Creamos un progressDialog para mostrar mientas se localiza a un
		 * amigo. Lo cerrará el hilo que gestiona la petición web para
		 * localizar.
		 */
		progreso = new ProgressDialog(this);
		progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		progreso.setMessage(Constantes.MSJ_OPERACION);
		progreso.setCancelable(true);

	}

	public void lanzarAddAmigos() {
		Intent intent = new Intent(this, AddAmigos.class);
		intent.putExtra("usuario", usuario);
		startActivityForResult(intent, Constantes.RESULTADO_ADD_AMIGOS);
	}

	public void lanzarDeleteAmigos() {
		Intent intent = new Intent(this, DeleteAmigos.class);
		intent.putExtra("usuario", usuario);
		startActivityForResult(intent, Constantes.RESULTADO_DELETE_AMIGOS);
	}

	public void lanzarInvitarAmigos() {
		Intent intent = new Intent(this, InvitarAmigos.class);
		intent.putExtra("usuario", usuario);
		startActivity(intent);
	}

	/**
	 * Método para localizar a tu amigo en el mapa. Si tienes la localización
	 * activada, trazará una línea recta hasta su posición y mostrará la
	 * distancia entre ambos.
	 * 
	 * @param v
	 */
	public void localizarAmigo(View v) {

		Button bt_localizar_amigo = (Button) findViewById(R.id.bt_localizar_amigo);

		HiloWebMostrarLocalizacion hiloWeb = new HiloWebMostrarLocalizacion(
				this, mapa, progreso);

		activarProveedores();
		hiloWeb.execute(bt_localizar_amigo.getTag().toString(), mejorLocaliz);
	}

	/**
	 * Envía una correo al usuario para que se conecte y poder ver su
	 * localización.
	 * 
	 * @param v
	 */
	public void escribirMensaje(View v) {

		Button bt_mandar_mensaje = (Button) findViewById(R.id.bt_mandar_mensaje);

		String amigo = bt_mandar_mensaje.getTag().toString();

		String mensaje = usuario + " pide que actives la localización.";

		hiloWeb = new HiloWeb(this);
		hiloWeb.execute(TipoOperacion.ESCRIBIR_MENSAJE, usuario, amigo, mensaje);
	}

	/**
	 * Muestra un relativeLayout con información para cada función del programa.
	 */
	public void lanzarAyuda() {

		if (infVisible) {
			layout_ayuda.setVisibility(View.GONE);
			infVisible = false;
		} else {
			layout_ayuda.setVisibility(View.VISIBLE);
			infVisible = true;
		}
	}

	/***
	 * Métodos relacionados con la gestión del mapa
	 */

	// Mtodos del ciclo de vida de la actividad

	@Override
	protected void onActivityResult(int requestcode, int resultcode, Intent arg2) {
		super.onActivityResult(resultcode, requestcode, arg2);
		if ((requestcode == Constantes.RESULTADO_DELETE_AMIGOS || requestcode == Constantes.RESULTADO_ADD_AMIGOS)
				&& resultcode == Activity.RESULT_OK) {

			hiloWebMostrarAmigosCon = new HiloWebMostrarAmigos(this,
					lista_conectados);

			hiloWebMostrarAmigosCon.execute(
					TipoOperacion.MOSTRAR_AMIGOS_ESTADO, usuario, "conectado");

			hiloWebMostrarAmigosDes = new HiloWebMostrarAmigos(this,
					lista_desconectados);

			hiloWebMostrarAmigosDes.execute(
					TipoOperacion.MOSTRAR_AMIGOS_ESTADO, usuario,
					"desconectado");
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		activarProveedores();

		actualizarEstado(mejorLocaliz);

		startService(new Intent(this, ServicioActualizacionListas.class));

	}

	@Override
	protected void onPause() {

		super.onPause();

		manejador.removeUpdates(this);

		stopService(new Intent(this, ServicioActualizacionListas.class));

	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		desconectar();
	}

	/**
	 * Guarda el número de conectados y desconectados y el nombre de cada uno de
	 * ellos.
	 */
	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);

		outState.putBoolean("infVisible", infVisible);
		outState.putBoolean("desconecta", desconecta);

		/*
		 * Guardamos la lista de conectados y el número de ellos.
		 */
		int num_con = lista_conectados.getCount();
		for (int i = 0; i < num_con; i++) {
			outState.putString("lista_conectados_" + i,
					(String) lista_conectados.getItemAtPosition(i));
		}

		outState.putInt("num_conectados", num_con);

		/*
		 * Guardamos la lista de desconectados y el número de ellos.
		 */
		int num_descon = lista_desconectados.getCount();
		for (int i = 0; i < num_descon; i++) {
			outState.putString("lista_desconectados" + i,
					(String) lista_desconectados.getItemAtPosition(i));
		}

		outState.putInt("num_desconectados", num_descon);

	}

	/**
	 * Recupera el número de conectados y desconectados y el nombre de cada uno
	 * de ellos.
	 */
	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);

		infVisible = savedInstanceState.getBoolean("infVisible");
		desconecta = savedInstanceState.getBoolean("desconecta");

		/*
		 * Recuperamos la lista de conectados y el número de ellos y
		 * actualizamos el listView de desconectados.
		 */

		cargarListaConectados(savedInstanceState);

		/*
		 * Recuperamos la lista de conectados y el número de ellos y
		 * actualizamos el listView de desconectados.
		 */

		cargarListaDesconectados(savedInstanceState);

	}

	/**
	 * Carga la lista de conectados.
	 * 
	 * @param savedInstanceState
	 * @param num_con
	 */
	public void cargarListaConectados(Bundle savedInstanceState) {

		int num_con = savedInstanceState.getInt("num_conectados");

		List<String> lista_con = new ArrayList<String>();

		for (int i = 0; i < num_con; i++) {
			lista_con
					.add(savedInstanceState.getString("lista_conectados_" + i));
		}

		lista_conectados.setAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, lista_con));
	}

	/**
	 * Carga la lista de desconectados.
	 * 
	 * @param savedInstanceState
	 * @param num_con
	 */
	public void cargarListaDesconectados(Bundle savedInstanceState) {

		int num_descon = savedInstanceState.getInt("num_desconectados");

		List<String> lista_descon = new ArrayList<String>();

		for (int i = 0; i < num_descon; i++) {
			lista_descon.add(savedInstanceState.getString("lista_desconectados"
					+ i));
		}

		lista_desconectados.setAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, lista_descon));
	}

	/**
	 * Crea el actionBar.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.action_bar_info, menu);
		return true;
	}

	/**
	 * Si se pulsa sobre una de los iconos del actionBar.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		int elementoPulsado = item.getItemId();

		switch (elementoPulsado) {
		case R.id.ayuda:
			lanzarAyuda();
			break;
		default:
			break;
		}

		return super.onOptionsItemSelected(item);
	}

	private void activarProveedores() {

		if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {

			manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER,

			Constantes.DOS_MINUTOS, Constantes.DISTANCIA_MIN, this);

			actualizaMejorLocaliz(manejador
					.getLastKnownLocation(LocationManager.GPS_PROVIDER));

		}

		if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {

			manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,

			Constantes.TIEMPO_MIN, Constantes.DISTANCIA_MIN, this);

			actualizaMejorLocaliz(manejador
					.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));

		}

	}

	private boolean actualizaMejorLocaliz(Location localiz) {

		if (localiz != null && (mejorLocaliz == null

		|| localiz.getAccuracy() < 2 * mejorLocaliz.getAccuracy()

		|| localiz.getTime() - mejorLocaliz.getTime() > Constantes.DOS_MINUTOS)) {

			Log.d(Constantes.TAG, "Nueva mejor localizacin");

			mejorLocaliz = localiz;

			return true;
		}
		return false;

	}

	public void verMiPosicion(View view) {

		actualizaMejorLocaliz(manejador
				.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));

		if (mejorLocaliz != null) {

			mapa.animateCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(
					mejorLocaliz.getLatitude(), mejorLocaliz.getLongitude()),
					15));
		}

	}

	@Override
	public void onMapClick(LatLng puntoPulsado) {

		mapa.addMarker(new MarkerOptions().position(puntoPulsado).

		icon(BitmapDescriptorFactory

		.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)));

	}

	@Override
	public void onLocationChanged(Location location) {

		if (actualizaMejorLocaliz(location)) {

			actualizarEstado(location);

			mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(
					location.getLatitude(), location.getLongitude()), 17));

		}

	}

	@Override
	public void onProviderDisabled(String proveedor) {

		activarProveedores();

	}

	@Override
	public void onProviderEnabled(String proveedor) {

		activarProveedores();

	}

	@Override
	public void onStatusChanged(String proveedor, int estado, Bundle extras) {

		activarProveedores();

	}

	/***
	 * Actualiza el estado del usuario(conectado/desconectado).
	 * 
	 * @param mejorLocaliz
	 */
	public void actualizarEstado(Location mejorLocaliz) {

		hiloWeb = new HiloWeb(this);

		if (mejorLocaliz != null) {
			hiloWeb.execute(TipoOperacion.ACTUALIZAR_ESTADO, usuario, estado,
					mejorLocaliz.getLatitude(), mejorLocaliz.getLongitude());
		} else {
			hiloWeb.execute(TipoOperacion.ACTUALIZAR_ESTADO, usuario, estado,
					0.0, 0.0);
		}

	}

	/**
	 * Actualiza el estado del usuario en la bd, para el servicio y elimina las
	 * notificaciones.
	 */
	public void desconectar() {

		HiloWeb hiloWeb = new HiloWeb(this);

		/*
		 * Guardamos una localización incorrecta.
		 */

		hiloWeb.execute(TipoOperacion.ACTUALIZAR_ESTADO, usuario,
				"desconectado", 0.0, 0.0);

		// Paramos los servicios
		stopService(new Intent(this, ServicioMensajeria.class));
		stopService(new Intent(this, ServicioActualizacionListas.class));

		nm.cancel(ID_NOTIFICACION);

	}

	/**
	 * Receptor de mensajes broadcast que recogerá mensajes del servicio de
	 * mensajería para mostrar los mensajes al usuario por medio de una
	 * notificación.
	 * 
	 * @author gatsu
	 * 
	 */
	public class ReceptorMensajes extends BroadcastReceiver {

		public static final String ACTION_RESP = "org.example.intentservice.intent.action.RESPUESTA_MENSAJE";

		public static final String ACTION_LISTA = "org.example.intentservice.intent.action.RESPUESTA_ACT_LIST";

		@Override
		public void onReceive(Context arg0, Intent intent) {

			if (intent.getAction() == ACTION_LISTA) {
				mostrarAmigos();
			} else {

				// Recogemos el mensaje de la intención pasada por el servicio.
				mensajes = intent.getStringArrayListExtra("mensajes");

				// Para que la notificación sea más grande
				NotificationCompat.InboxStyle inboxStyle = new NotificationCompat.InboxStyle();

				inboxStyle.setBigContentTitle("Nuevos Mensajes Buddy");

				for (String mensaje : mensajes) {
					inboxStyle.addLine(mensaje);
				}

				NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(
						Home.this)
						.setSmallIcon(R.drawable.ic_launcher)
						.setContentText(mensajes.toString())
						.setContentTitle("Nuevos Mensajes Buddy")
						.setSound(
								Uri.parse("android.resource://"
										+ getPackageName() + "/"
										+ R.raw.silvido));

				mBuilder.setStyle(inboxStyle);

				Intent i = new Intent(Home.this, Home.class);
				i.putStringArrayListExtra("mensajes",
						(ArrayList<String>) mensajes);
				PendingIntent pendingIntent = PendingIntent.getActivity(
						Home.this, 0, i, 0);
				mBuilder.setContentIntent(pendingIntent);

				nm.notify(ID_NOTIFICACION, mBuilder.build());

			}

		}
	}

	/**
	 * Lo llamaremos cuando abramos una notificación.
	 */
	@Override
	protected void onNewIntent(Intent intent) {
		super.onNewIntent(intent);

		/*
		 * Si volvemos a llamar a la actividad, será desde la notificación, y
		 * marcaremos como leído el mensaje en la base de datos.
		 */

		nm.cancel(ID_NOTIFICACION);

		hiloWeb = new HiloWeb(this);
		hiloWeb.execute(TipoOperacion.MARCAR_LEIDOS, usuario);

		Toast toast = Toast
				.makeText(this, "Mensajes leídos", Toast.LENGTH_LONG);
		toast.setGravity(Gravity.CENTER, 0, 0);
		toast.show();

	}

	/***
	 * Muestra los amigos del usuario en las listView.
	 */
	public void mostrarAmigos() {

		/*
		 * Sacamos la información a través de la actividad.
		 */

		/*
		 * Llamamos al servicio mostrar_amigos.php, y el método devuelve los
		 * amigos del usuario en una lista.
		 */
		hiloWebMostrarAmigosCon = new HiloWebMostrarAmigos(this,
				lista_conectados);

		hiloWebMostrarAmigosCon.execute(TipoOperacion.MOSTRAR_AMIGOS_ESTADO,
				usuario, "conectado");

		hiloWebMostrarAmigosDes = new HiloWebMostrarAmigos(this,
				lista_desconectados);

		hiloWebMostrarAmigosDes.execute(TipoOperacion.MOSTRAR_AMIGOS_ESTADO,
				usuario, "desconectado");

	}
}
