package org.example.buddy;

import org.example.serviciosweb.HiloWebIngreso;
import org.example.serviciosweb.ServicioWebUsuarios;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

public class Ingreso extends Activity {

	private String usuario, passw;
	private EditText edit_usuario;
	private EditText edit_passw;
	private HiloWebIngreso hiloWeb;
	private SharedPreferences preferencias;
	private ProgressDialog progreso;
	private boolean estaHiloEjecutando;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.ingreso_layout);

		edit_usuario = (EditText) findViewById(R.id.edit_usuario);
		edit_passw = (EditText) findViewById(R.id.edit_passw);
		Button bt_inicio = (Button) findViewById(R.id.bt_inicio);
		Button bt_registro = (Button) findViewById(R.id.bt_registro);

		bt_inicio.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				comprobarCredenciales();
			}
		});

		bt_registro.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				lanzarRegistro();
			}
		});

		preferencias = getSharedPreferences(Constantes.PREFERENCIAS,
				Context.MODE_PRIVATE);

		usuario = preferencias.getString("usuario", "");
		passw = preferencias.getString("passw", "");

		if (!usuario.equals("") && !passw.equals("")) {

			edit_usuario.setText(usuario);
			edit_passw.setText(passw);
		}

		if (savedInstanceState != null) {
			estaHiloEjecutando = savedInstanceState.getBoolean("estahilo");
		}

		progreso = new ProgressDialog(this);
		progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		progreso.setMessage(Constantes.MSJ_OPERACION);
		progreso.setCancelable(true);

	}

	public void comprobarCredenciales() {

		Toast toast;
		usuario = edit_usuario.getText().toString();
		passw = edit_passw.getText().toString();

		if (usuario.compareTo("") == 0 || passw.compareTo("") == 0) {
			toast = Toast.makeText(this,
					"Rellena los campos usuario y contraseña",
					Toast.LENGTH_LONG);
			toast.setGravity(Gravity.CENTER, 5, 5);
			toast.show();
			return;
		}

		hiloWeb = new HiloWebIngreso(this, progreso);

		hiloWeb.execute(usuario, passw);

		estaHiloEjecutando = true;

	}

	@Override
	protected void onResume() {
		super.onResume();
		seleccionarServidor();
	}

	public void lanzarRegistro() {
		startActivity(new Intent(this, Registro.class));
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);

		outState.putBoolean("estahilo", estaHiloEjecutando);

	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);

		if (savedInstanceState.containsKey("estahilo")) {
			if (savedInstanceState.getBoolean("estahilo"))
				progreso.show();
		}
	}

	/**
	 * Crea el actionBar.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.action_bar_config, menu);
		return true;
	}

	/**
	 * Si se pulsa sobre una de los iconos del actionBar.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		int elementoPulsado = item.getItemId();

		switch (elementoPulsado) {
		case R.id.config:
			lanzarPreferencias();
			break;
		default:
			break;
		}

		return super.onOptionsItemSelected(item);
	}

	public void lanzarPreferencias() {
		startActivity(new Intent(this, Preferencias.class));
	}

	/**
	 * Método privado que usará la actividad para saber como guardar las
	 * puntuaciones
	 */
	private void seleccionarServidor() {
		SharedPreferences pref = this.getSharedPreferences(
				"org.example.buddy_preferences", Context.MODE_PRIVATE);

		String servidor = pref.getString("servidor",
				"http://android-upv.esy.es/usuarios_Buddy/");
		ServicioWebUsuarios.setHosting(servidor);

	}

}
