package org.example.serviciosweb;

import java.util.List;

import org.example.buddy.Constantes;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.graphics.Color;
import android.location.Location;
import android.os.AsyncTask;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;

public class HiloWebMostrarLocalizacion extends
		AsyncTask<Object, Void, List<Double>> {

	private Activity actividad;
	private GoogleMap mapa;
	private String usuario;
	private Location location;
	private ProgressDialog progreso;

	public HiloWebMostrarLocalizacion(Activity actividad, GoogleMap mapa,
			ProgressDialog progreso) {
		this.actividad = actividad;
		this.mapa = mapa;
		this.progreso = progreso;
	}

	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		if (progreso != null) {
			progreso.show();

			progreso.setOnCancelListener(new OnCancelListener() {

				@Override
				public void onCancel(DialogInterface dialog) {
					cancel(true);

				}
			});

		}
	}

	@Override
	protected List<Double> doInBackground(Object... arg) {

		usuario = (String) arg[0];
		location = (Location) arg[1];

		if (!isCancelled()) {
			return ServicioWebUsuarios.mostrarLocalizacion(usuario);
		}
		return null;
	}

	@Override
	protected void onCancelled() {
	}

	@Override
	protected void onPostExecute(List<Double> result) {
		super.onPostExecute(result);

		if (result != null && result.size() > 0) {

			try {

				if (mapa != null) {

					double latitud_amigo = result.get(0);
					double longitud_amigo = result.get(1);

					/*
					 * Si la latitud es diferente de 0.0, es decir, de una
					 * localización por defecto vacía, no hacemos nada.
					 */
					if (latitud_amigo != 0.0 && longitud_amigo != 0.0) {

						mapa.clear();

						mapa.addMarker(
								new MarkerOptions().position(new LatLng(
										latitud_amigo, longitud_amigo)))
								.setTitle("Localización de " + usuario);

						mapa.animateCamera(CameraUpdateFactory.newLatLngZoom(
								new LatLng(result.get(0), result.get(1)), 15));

						/*
						 * Si tenemos la localización activada, trazaremos una
						 * línea recta hasta el objectivo(amigo) y calcularemos
						 * la distancia en kilómetros que hay entre nosotros.
						 */
						if (location != null) {

							double latitud_actual = location.getLatitude();
							double longitud_actual = location.getLongitude();

							PolylineOptions lineas = new PolylineOptions().add(
									new LatLng(result.get(0), result.get(1)))
									.add(new LatLng(location.getLatitude(),
											location.getLongitude()));

							float[] results = new float[1];
							Location.distanceBetween(latitud_amigo,
									longitud_amigo, latitud_actual,
									longitud_actual, results);

							Toast toast = Toast.makeText(actividad,
									"Distancia a tu amigo: \n" + results[0]
											/ 1000 + " kilómetros", 5000);
							toast.show();

							lineas.width(8);
							lineas.color(Color.RED);

							mapa.addPolyline(lineas);
						}

					}// Fin si latitud_amigo

					/*
					 * Si localización es 0.0, no tiene la localización
					 * activada.
					 */

					else {
						Toast toast = Toast.makeText(actividad,
								"Tu amigo no tiene la localización activada",
								Toast.LENGTH_LONG);
						toast.setGravity(Gravity.CENTER, 0, 0);
						toast.show();
					}

				}

			} catch (Exception e) {
				Log.e("Buddy_manual", Constantes.MSJ_ERROR_HILOS);
			}

		}

		if (progreso != null) {
			progreso.dismiss();
		}

	}
}
